/**
 * @file
 * @brief EMBuild-side binding interface.
 *
 * @date 12.06.2010
 * @author Eldar Abusalimov
 */

#ifndef MOD_EMBUILD_H_
#define MOD_EMBUILD_H_

#include <impl/mod/embuild.h>

/**
 * Defines a new mod. For the new mod the @link #mod corresponding @endlink
 * structure is allocated. Also two section-driven arrays are defined for the
 * mod#provides and mod#requires lists.
 *
 * @param s_mod
 *   The variable name used later to access the mod structure with #MOD_PTR(),
 *   to define dependencies with #MOD_DEP_DEF(), or to bind mod-specific
 *   @link #__mod_info information @endlink using #MOD_INFO_DEF() macro and its
 *   derivatives. For the code generated by EMBuild this argument must be the
 *   same as provided by the __EMBUILD_MOD__ macro for each mod at compilation
 *   time.
 * @param s_mod_package
 *   The package variable name used in #MOD_PACKAGE_DEF().
 * @param mod_name
 *   A string containing the mod name. The name of the current mod (referenced
 *   by __EMBUILD_MOD__) is available at compilation time using #MOD_SELF_NAME
 *   macro.
 * @param brief
 *   A one-line string containing brief description of the mod.
 * @param details
 *   A string with detailed description.
 */
#define MOD_DEF(s_mod, s_mod_package, mod_name, brief, details) \
		__MOD_DEF(s_mod, s_mod_package, mod_name, brief, details)

/**
 * Defines a new dependency between two specified modules.
 *
 * @param s_mod symbol name of the module which depends on @c s_dep
 * @param s_dep symbol name of the module on which @c s_mod depends
 */
#define MOD_DEP_DEF(s_mod, s_dep) \
		__MOD_DEP_DEF(s_mod, s_dep)

/**
 * Defines a new package.
 *
 * @param s_package the variable name
 * @param package_name a string containing the package name
 */
#define MOD_PACKAGE_DEF(s_package, package_name) \
		__MOD_PACKAGE_DEF(s_package, package_name)


#endif /* MOD_EMBUILD_H_ */
